//+------------------------------------------------------------------+
//|                                                 CL_Ranges_V3.mq5 |
//|                                           Copyright 2023, Igazi. |
//|                              https://www.mql5.com/en/users/Igazi |
//+------------------------------------------------------------------+
#property copyright "Copyright 2023, Igazi"
#property link      "https://www.mql5.com/en/users/Igazi"
#property indicator_chart_window


#property indicator_buffers 4
#property indicator_plots 4

input datetime inpDateInit = D'2023.01.01 00:00';  // Date/Time Init
input int      inpPeriod   = 3;                    // Period
input color    inpColor    = clrPurple;            // Box Colour
input bool     FillBody    = false;                // Fill Box Body
input int      inpWidth    = 1;                    // Line Width
input bool     ShowWicks = true;                   // Show Wicks


string pre = "";

double cnt[], h[], l[], box[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit() {
//--- indicator buffers mapping
   ArraySetAsSeries(h,true);
   ArraySetAsSeries(l,true);
   ArraySetAsSeries(box,true);
   ArraySetAsSeries(cnt,true);
   
   SetIndexBuffer(0,h,INDICATOR_DATA);
   SetIndexBuffer(1,l,INDICATOR_DATA);
   SetIndexBuffer(2,box,INDICATOR_DATA);
   SetIndexBuffer(3,cnt,INDICATOR_DATA);
   
   for(int i=0; i<4; i++) PlotIndexSetInteger(i,PLOT_DRAW_TYPE,DRAW_NONE);
   
   pre = "cl_" + (string)inpPeriod + "_";
   
   if(!ShowWicks){
      DeleteAllObjects("line");
      //Print("deleted");
   }
   DeleteAllObjectsBut(ConvertTimeFrame(Period()));
   //Print("start");
//---
   return(INIT_SUCCEEDED);
}

void OnDeinit(const int reason) {
   Comment("");
   DeleteAllObjects("cl_");
   //Print("exit");

}


//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]) {
//---
   static int last_tf = 0;
   static string last_symbol = "";
   

   
   if(rates_total<=0) return(0);
   if(prev_calculated<0) return(0);
   
   ArraySetAsSeries(time,true);
   ArraySetAsSeries(close,true);
   
   int calculated = prev_calculated;
   
   if(last_tf!=PeriodSeconds() || last_symbol!=_Symbol) {
      calculated = 0;
      last_tf = PeriodSeconds();
      last_symbol = _Symbol;
   }
   
   int period = fmax(1,inpPeriod);
   int bar_shift = inpDateInit<TimeCurrent()? iBarShift(_Symbol,PERIOD_CURRENT,inpDateInit) : -1;
   int limit = MathMax(bar_shift,rates_total - calculated + 1 + period*2);
   
   
   for(int i=limit; i>=0; i--) {
      if(i>=rates_total) continue;
      
      if(i==(rates_total-1) || bar_shift<1 || i>=bar_shift) {
         h[i] = l[i] = box[i] = cnt[i] = 0;
         continue;
      }
      
      //Print(i+" "+rates_total+ " " + calculated);

      
      cnt[i] = cnt[i+1] + 1;
      box[i] = box[i+1];
      
      h[i] = fmax(h[i+1],close[i]);
      l[i] = fmin(l[i+1],close[i]);
      
      if(cnt[i]>period) cnt[i] = 1;
      
      if(cnt[i]==1) {
         h[i] = l[i] = close[i];
         box[i] += 1;
         CheckBox(i,period);
      }
      else {
         h[i] = fmax(h[i],close[i]);
         l[i] = fmin(l[i],close[i]);
      }
      
   }
   
   ChartRedraw();
//--- return value of prev_calculated for next call
   return(rates_total);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CheckBox(int i, int period) {
//---
   if(box[i]<=1) return;
   double high = h[i+1];
   double low  = l[i+1];
   datetime time1 = iTime(_Symbol,PERIOD_CURRENT,i+period);
   datetime time2 = iTime(_Symbol,PERIOD_CURRENT,i+1);
   datetime time3 = iTime(_Symbol,PERIOD_CURRENT,i+1+int(period/2));
   string name_box = pre +" "+Symbol()+ " "+ConvertTimeFrame(Period())+" "+"box_" + (string)(int)time1;
   string name_line = pre +" "+Symbol()+ " "+ConvertTimeFrame(Period())+" "+ "line_" + (string)(int)time3;
   
   //ObjectDelete(0,name_box);
   //ObjectDelete(0,name_line);
   
   DrawBox(name_box,time1,low,time2,high);
   
   if(box[i]<=2) return;
   double prev_high = h[i+1+period];
   double prev_low = l[i+1+period];
   
   if(low>prev_high){
      if(ShowWicks){
         DrawBox(name_line,time3,prev_high,time3,low);
      }
   }
   if(high<prev_low){
      if(ShowWicks){
         DrawBox(name_line,time3,prev_low,time3,high);
      }
   }
//---
}

void DrawBox(string name, datetime time1, double price1, datetime time2, double price2) {
   long chart_ID = 0;
   ObjectCreate(chart_ID,name,OBJ_RECTANGLE,0,time1,price1,time2,price2);
   ObjectSetInteger(chart_ID,name,OBJPROP_COLOR,inpColor);
   ObjectSetInteger(chart_ID,name,OBJPROP_STYLE,STYLE_SOLID);
   ObjectSetInteger(chart_ID,name,OBJPROP_WIDTH,fmax(1,inpWidth));
   if(StringFind(name,"box")>=0){
      ObjectSetInteger(chart_ID,name,OBJPROP_FILL,FillBody);
   }
   else{
      ObjectSetInteger(chart_ID,name,OBJPROP_FILL,false);
   }
   ObjectSetInteger(chart_ID,name,OBJPROP_BACK,false);
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTED,false);
   ObjectSetInteger(chart_ID,name,OBJPROP_HIDDEN,true);
   ObjectSetInteger(chart_ID,name,OBJPROP_ZORDER,0);
}
//+------------------------------------------------------------------+


void DeleteAllObjects(string text)
  {
   for(int i = ObjectsTotal(0) -1; i >=0; i--)
     {
      if(StringFind(ObjectName(0,i),text)>=0)
        {
         ObjectDelete(0,ObjectName(0,i));
        }
     }
  }


void DeleteAllObjectsBut(string text)
  {
   for(int i = ObjectsTotal(0) -1; i >=0; i--)
     {
      if(StringFind(ObjectName(0,i),text)<0||StringFind(ObjectName(0,i),Symbol())<0)
        {
         ObjectDelete(0,ObjectName(0,i));
        }
     }
  }
  
  
  
string ConvertTimeFrame(int tf){
   switch(tf)                                  // Header of the 'switch'
     {                                            // Start of the 'switch' body
      case PERIOD_M1 : return("M1");
      case PERIOD_M5 : return("M5");
      case PERIOD_M15 : return("M15");
      case PERIOD_M30 : return("M30");
      case PERIOD_H1 : return("H1");
      case PERIOD_H4 : return("H4");
      case PERIOD_D1 : return("D1");
      case PERIOD_W1 : return("W1");
      case PERIOD_MN1 : return("MN");
     }
     return("");
}  